// ============================================================================
// Project:	Deductive Database
// Filename:	idiv.h
// Purpose:	Macros for rounded integer division and percent
// Last Change:	04.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file  idiv.h
 * \brief Macros for rounded integer division and percent
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	IDIV_INCLUDED
#define IDIV_INCLUDED


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif


//=============================================================================
// Defined Types and Macros:
//=============================================================================

// Note that all macros work only for non-negative numbers.
// For negative numbers, C and C++ do not completely prescribe the result
// of integer division. It would be more complicated to get a portable result.

//-----------------------------------------------------------------------------
// IDIV_ROUND:
//-----------------------------------------------------------------------------

/**
 * Integer division, round result
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_ROUND(X,Y)	(((X)+(Y)/2)/(Y))

//-----------------------------------------------------------------------------
// IDIV_CEILING:
//-----------------------------------------------------------------------------

/**
 * Integer division, round up.
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_CEILING(X,Y)	(((X)+(Y)-1)/(Y))

//-----------------------------------------------------------------------------
// IDIV_WHOLE:
//-----------------------------------------------------------------------------

/**
 * Integer division with one decimal place, whole part.
 *
 * Note that this works only for non-negative numbers.
 */

#define IDIV_WHOLE(X,Y)	((((X)*10+(Y)/2)/(Y))/10)

//-----------------------------------------------------------------------------
// IDIV_TENTH:
//-----------------------------------------------------------------------------

/**
 * Integer division with one decimal place, tenth.
 *
 * Note that this works only for non-negative numbers.
 */

//#define IDIV_TENTH(X,Y)	((((X)*10+(Y)/2)/(Y))%10)
#define IDIV_TENTH(X,Y)	(((((X)-IDIV_WHOLE((X),(Y))*(Y)) * 10 + (Y)/2)) / (Y))
	// In this way, we do not have to multiply X by 10,
	// which is important if X is large (near the integer limit).

//-----------------------------------------------------------------------------
// IDIV_PERCENT:
//-----------------------------------------------------------------------------

/**
 * Compute Percentage, round result.
 */

#define IDIV_PERCENT(X,Y)	(((X)*100+((Y)/2))/(Y))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

