// ============================================================================
// Project:	Deductive Database
// Filename:	min.h
// Purpose:	Minimum-Function
// Last Change:	04.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file  min.h
 * \brief An include file with the minimum function (as template or macro).
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	MIN_INCLUDED
#define MIN_INCLUDED


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif


//=============================================================================
// Template Function:
//=============================================================================


//-----------------------------------------------------------------------------
// Minium:
//-----------------------------------------------------------------------------

/**
 * \def   MIN(X, Y)
 * \brief Minimum of \a X and \a Y.
 *
 * Note: The minimum function might be implemented as a macro.
 * The arguments should have no side effects.
 */

//#define MIN(X,Y)	(((X)<=(Y))?(X):(Y))

/**
 * Minimum of \a X and \a Y.
 *
 * This was a macro, but I converted it to an inline function template.
 * It returns the smaller of the two arguments.
 */


template<class T>
inline T MIN(T x, T y) {
	if(x <= y)
		return x;
	else
		return y;
}


//=============================================================================
// End of Include File:
//=============================================================================

#endif

