// ============================================================================
// Project:	Deductive Database
// Filename:	set_2s.h
// Purpose:	Set of rows with two int columns (relation with binding bb)
// Last Change:	04.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2016-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Set of rows with two int columns (relation with binding "bb").
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	SET_2S_INCLUDED
#define SET_2S_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

#ifndef ROW_2_INCLUDED
#include "row_2.h"
#endif

#ifndef SET_S_INCLUDED
#include "set_s.h"
#endif


//=============================================================================
// Set of Rows with two int columns / Relation with binding pattern bb:
//=============================================================================

class set_2s_c: public set_s_c<row_2_c> {

	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	// Constructor:
	set_2s_c(str_t set_name);

	// Destructor:
	~set_2s_c();

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	set_2s_c(const set_2s_c& obj);			// Not implemented
	set_2s_c& operator=(const set_2s_c& obj);	// Not implemented


//=============================================================================
// (Object) Methods:
//=============================================================================

	public:

//-----------------------------------------------------------------------------
// insert: Insert an element into the set (returns true if successful).
//-----------------------------------------------------------------------------

	// This method returns true if the row was not already in the set
	// and was successfully inserted (i.e. if it is not a duplicate).
	// It returns false if the row was already in the set (duplicate).
	// It also returns false if it was impossible to insert the element
	// because there was not enough memory.

	inline bool insert(int col1, int col2) {
		return set_s_c<row_2_c>::insert(row_2_c(col1, col2));
	}

//-----------------------------------------------------------------------------
// contains: Check whether an element is contained in the set.
//-----------------------------------------------------------------------------

	inline bool contains(int col1, int col2) const {
		return set_s_c<row_2_c>::contains(row_2_c(col1, col2));
	}

//=============================================================================
// End of Class Template:
//=============================================================================

};


//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef set_2s_c *set_2s_t;

//-----------------------------------------------------------------------------
// Define null pointer:
//-----------------------------------------------------------------------------

#define SET_2S_NULL (static_cast<set_2s_t>(0))


//=============================================================================
// End of Include File:
//=============================================================================

#endif

