// ============================================================================
// Project:	Deductive Database
// Filename:	bench_j1axy_2.h
// Purpose:	Benchmark Join1, a(X,Y), Ver. 2: no goto, dupcheck, bitmap
// Last Change:	17.09.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2015-2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Join1 Benchmark a(X, Y),
 * 	Version 2 (Full duplicate check, no goto, bitmap set implementation).
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_J1AXY_2_INCLUDED
#define BENCH_J1AXY_2_INCLUDED


//=============================================================================
// This Benchmark is optional:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#if	VER_BENCH_J1AXY_2


//=============================================================================
// Used Types and Macros:
//=============================================================================


#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

#ifndef LIST_2_INCLUDED
#include "../rel/list_2.h"
#endif

#ifndef SET_TT_INCLUDED
#include "../rel/set_tt.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "../rel/rel_n_n.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif


//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_J1AXY_2_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_J1AXY_2_MAGIC = 0x4A31410AL;	// 'J1A\n'


//=============================================================================
// Benchmark Join1, a(X,Y), Ver. 2: no goto, with duplicate check, bitmap-set
//=============================================================================

class bench_j1axy_2_c : public bench_c {
	public:


//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_j1axy_2_c();

	// Destructor:
	~bench_j1axy_2_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return "Join 1, a(X,Y), Ver. 2: no goto, with bitmap dupcheck";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;


//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_j1axy_2_c(const bench_j1axy_2_c& obj);
	bench_j1axy_2_c& operator=(const bench_j1axy_2_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#if	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_J1AXY_2_MAGIC".
#endif


#if	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Data storage for EDB relations:
//-----------------------------------------------------------------------------

	rel_n_n_c c2_bf;
	list_2_c c3_ff;
	rel_n_n_c c4_bf;
	list_2_c d1_ff;
	rel_n_n_c d2_bf;


//-----------------------------------------------------------------------------
// Relation for storing derived b1 tuples (for rule with two IDB lit):
//-----------------------------------------------------------------------------

	rel_n_n_c b1_fb;
		// Note that columns 1 and 2 are exchanged,
		// because c1 is always input to the mapping, and c2 output.

//-----------------------------------------------------------------------------
// Sets for duplicate checks:
//-----------------------------------------------------------------------------

	set_tt_c b1_bb;
	set_tt_c b2_bb;
	set_tt_c c1_bb;
	set_tt_c a_bb;

//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Conditional Compilation of this Test (VER_BENCH_J1AXY_2):
//=============================================================================

#endif


//=============================================================================
// End of Include File:
//=============================================================================

#endif

