// ============================================================================
// Project:	Deductive Database
// Filename:	bench_tcff_5.h
// Purpose:	Bottom-Up Evaluation for tc(X,Y), Ver 5: BAM
// Last Change:	11.08.2017
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 2017 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


/**
 * \file
 * \brief Transitive Closure Benchmark tc(X,Y) (Binding Pattern "ff"),
 *	Version 5 (with Abstract Machine)
 */


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	BENCH_TCFF_5_INCLUDED
#define BENCH_TCFF_5_INCLUDED


//=============================================================================
// This Benchmark is optional:
//=============================================================================

#ifndef VER_INCLUDED
#include "../base/ver.h"
#endif

#if	VER_BENCH_TCFF_5


//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef STR_INCLUDED
#include "../base/str.h"
#endif

#ifndef CHECK_INCLUDED
#include "../base/check.h"
#endif

#ifndef LIST_2_INCLUDED
#include "../rel/list_2.h"
#endif

#ifndef CUR_2_INCLUDED
#include "../rel/cur_2.h"
#endif

#ifndef SET_2_INCLUDED
#include "../rel/set_2.h"
#endif

#ifndef REL_N_N_INCLUDED
#include "../rel/rel_n_n.h"
#endif

#ifndef BAM_INCLUDED
#include "../emu/bam.h"
#endif

#ifndef BENCH_INCLUDED
#include "bench.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// BENCH_TCFF_5_MAGIC: Magic number (identifies objects of this class).
//-----------------------------------------------------------------------------

static const long BENCH_TCFF_5_MAGIC = 0x5443460AL;	// 'TCF\n'


//=============================================================================
// Bottom-Up Evalutation (DBLP Benchmark) - Ver. 5: BAM
//=============================================================================

class bench_tcff_5_c : public bench_c {
	public:

//-----------------------------------------------------------------------------
// Constructor, Destructor:
//-----------------------------------------------------------------------------

	public:

	// Constructor:
	bench_tcff_5_c();

	// Destructor:
	~bench_tcff_5_c();

//-----------------------------------------------------------------------------
// (Object) Methods:
//-----------------------------------------------------------------------------

	// load: Load data file with EDB facts.
	bool load(str_t filename);

	// eval: Do bottom-up evaluation and read result over cursor interface.
	long eval();

	// name: Name of this benchmark.
	str_t name() const {
		return
		"tc(X,Y) Bench., Ver.5: Abstract Machine";
	}

	// file: List of data files.
	str_t file(int i) const;

	// result: List of correct results of eval method.
	long result(int i) const;

	// Open cursor (note: load() must be called before this method).
	void open();

	// fetch: Get next tuple, returns false if there is no further tuple.
	inline bool fetch() {
		return result_cursor.fetch();
	}

	// c1: Return value for column 1 of current row in cursor.
	inline int c1() {
		return result_cursor.c1();
	}

	// c2: Return value for column 2 of current row in cursor.
	inline int c2() {
		return result_cursor.c2();
	}

	// error: Did an error occur?
	inline bool error() {
		if(par_ff.mem_err())
			return true;
		if(par_fb.mem_err())
			return true;
		if(tc_bb.mem_err())
			return true;
		if(result_list.mem_err())
			return true;
		return Error;
	}

	// close: Close cursor.
	inline void close() {
		result_cursor.close();
	}

//-----------------------------------------------------------------------------
// Copy-constructor and assignment operator are not supported for this class:
//-----------------------------------------------------------------------------

	private:

	// These methods are not implemented:
	bench_tcff_5_c(const bench_tcff_5_c& obj);
	bench_tcff_5_c& operator=(const bench_tcff_5_c& obj);


//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#if	VER_DEBUG
	// Integrity check:
	public:
	str_t	check() const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "BENCH_TCFF_5_MAGIC".
#endif


#if	VER_DUMP
	public:
	// Display data structure:
	void dump(str_t headline = STR_NULL) const;
#endif

//-----------------------------------------------------------------------------
// Auxiliary Methods:
//-----------------------------------------------------------------------------

	private:

	bool bam_prog();

//-----------------------------------------------------------------------------
// Private Object Members:
//-----------------------------------------------------------------------------

	private:

//-----------------------------------------------------------------------------
// Abstract machine:
//-----------------------------------------------------------------------------

	bam_c bam;

//-----------------------------------------------------------------------------
// Relations for EDB body literals:
//-----------------------------------------------------------------------------

	list_2_c  par_ff;
	rel_n_n_c par_fb;

//-----------------------------------------------------------------------------
// Relations for duplicate check (IDB predicates):
//-----------------------------------------------------------------------------

	set_2_c tc_bb;

//-----------------------------------------------------------------------------
// Result list and cursor:
//-----------------------------------------------------------------------------

	list_2_c result_list;
	cur_2_c  result_cursor;

//-----------------------------------------------------------------------------
// Did an error occur?
//-----------------------------------------------------------------------------

	bool Error;


//=============================================================================
// End of Class:
//=============================================================================

};


//=============================================================================
// End of Conditional Compilation of this Benchmark (VER_BENCH_TCFF_5):
//=============================================================================

#endif


//=============================================================================
// End of Include File:
//=============================================================================

#endif

